<?php
/**
 * Pro Class
 *
 * This file is used to modify any Pro related filtes, hooks & modifiers
 *
 * @package Layers
 * @since Layers 1.0
 */

class Layers_Pro {

	private static $instance;

	/**
	*  Get Instance creates a singleton class that's cached to stop duplicate instances
	*/
	public static function get_instance() {
		if ( ! self::$instance ) {
			self::$instance = new self();
			self::$instance->init();
		}
		return self::$instance;
	}

	/**
	*  Construct empty on purpose
	*/

	private function __construct() {}

	/**
	*  Init behaves like, and replaces, construct
	*/

	public function init(){

		// Run the version checker
		$this->layers_version_check();

		// Check for Layers as well as WooCommerce
		add_action( 'admin_notices', array( $this, 'activate_admin_notice' ) );

		if( FALSE !== $this->update_required || 'layerswp' != get_template() ) return;

		// Send the user to onboarding upon activation
		add_action( 'init' , array( $this, 'onboard_redirect') );

		// Includes
		require_once( LAYERS_PRO_DIR . 'includes/helpers.php' ); // Customizer Controls
		require_once( LAYERS_PRO_DIR . 'controls/controls.php' ); // Customizer Controls
		require_once( LAYERS_PRO_DIR . 'widgets/post-carousel.php' ); // Post Carousel Widget
		require_once( LAYERS_PRO_DIR . 'widgets/call-to-action.php' ); // Call To Action Widget
		require_once( LAYERS_PRO_DIR . 'widgets/tabs.php' ); // Tabs Widget
		require_once( LAYERS_PRO_DIR . 'widgets/accordion.php' ); // Accordian Widget
		require_once( LAYERS_PRO_DIR . 'widgets/social-icons.php' ); // Social Icons Widget

		// Add Onboarding Page
		add_action( 'admin_menu', array( $this, 'add_submenu_page' ), 60 );

		// Add Styles & Scripts
		add_filter( 'wp_enqueue_scripts', array( $this , 'enqueue_scripts' ), 30 );
		add_filter( 'admin_enqueue_scripts', array( $this , 'admin_enqueue_scripts' ) );

		// Add new controls to the slider widget if Layers is running 1.2.10
		if( !version_compare( $this->layers_meta->get( 'Version' ), '1.2.10', '<' ) ){
			add_filter( 'layers_design_bar_background_slide_item_elements' , array( $this, 'slider_elements' ), 10, 2 );
			add_action( 'layers_before_slider_widget_item_inner' , array( $this, 'slider_video_bg' ), 10, 3 );
		}
	}

	/**
	* Activate Pro admin notice
	*/
	public function activate_admin_notice(){
		global $blog_id;
		$themes = wp_get_themes( $blog_id );
		if( 'layerswp' !== get_template() ){ ?>
			<div class="updated is-dismissible notice">
				<p><?php _e( sprintf( "Layers is required to use Pro. <a href=\"%s\" target=\"_blank\">Click here</a> to get it.", ( isset( $themes['layerswp'] ) ? admin_url( 'themes.php?s=layerswp' ) : "http://www.layerswp.com" ) ), 'layers-storekit' ); ?></p>
			</div>
		<?php } elseif( FALSE !== $this->update_required ) { ?>
			<div class="updated is-dismissible notice">
				<p><?php _e( sprintf( "Pro requires Layers Version ". $this->update_required .". <a href=\"%s\" target=\"_blank\">Click here</a> to get the Layers Updater.", "http://www.layerswp.com/download/layers-updater" ), 'layers-storekit' ); ?></p>
			</div>
	<?php }
	}

	/**
	* Layers Min Version Checker
	*/
	public function layers_version_check(){

		$this->layers_meta = wp_get_theme( 'layerswp' );

		if( version_compare( $this->layers_meta->get( 'Version' ), LAYERS_PRO_REQUIRED_VERSION, '<' ) ){
			$this->update_required = LAYERS_PRO_REQUIRED_VERSION;
		} else {
			$this->update_required = FALSE;
		}
	}

	/**
	* Activation Hook
	*/
	public static function activate(){

		// Set Activation Transient
		set_transient( 'layers_pro_activated', 1, 30 );

		// if ( ( $mod = get_theme_mod( 'layers-header-background-color' ) ) && FALSE == get_theme_mod( 'layers-buttons-primary-background-color' ) ) {
		// 	_log( set_theme_mod( 'layers-buttons-primary-background-color', $mod ) );
		// }
	}

	/**
	* Redirect Users to onboarding upon activation
	*/
	public function onboard_redirect(){
		 // Only do this if the user can activate plugins
		if ( ! current_user_can( 'manage_options' ) )
			return;

		// Don't do anything if the transient isn't set
		if ( ! get_transient( 'layers_pro_activated' ) )
			return;

		// Redirect to onboarding - disabled for now!
		// wp_redirect( admin_url( 'admin.php?page=layers-pro-get-started' ) );
	}


	/**
	* Add Sub Menu Page to the Layers Menu Item
	*/
	public function add_submenu_page(){
		add_submenu_page(
			'layers-dashboard',
			__( 'Pro Help' , 'layers-pro' ),
			__( 'Pro Help' , 'layers-pro' ),
			'edit_theme_options',
			'layers-pro-get-started',
			array( $this, 'load_onboarding' )
		);
	}

	public function load_onboarding(){
		// Include Partials, we're using require so that inside the partial we can use $this to access the header and footer
		require LAYERS_PRO_DIR .'includes/onboarding.php';
	}

	/**
	*  Enqueue Admin Scripts & Styles
	*/

	public function admin_enqueue_scripts(){

		// Enqueue Script
		wp_enqueue_script(
			LAYERS_PRO_SLUG . '-admin',
			LAYERS_PRO_URI . 'assets/js/admin.js',
			array( 'jquery' ),
			LAYERS_PRO_VER,
			true
		);
		wp_localize_script(
			LAYERS_PRO_SLUG . '-admin' ,
			'layers_pro_params',
			array(
				'social_networks' => layers_pro_get_social_networks( 'native-with-values' ),
			)
		);

		// Enqueue CSS
		wp_enqueue_style(
			LAYERS_PRO_SLUG . '-admin',
			LAYERS_PRO_URI . 'assets/css/admin.css',
			array(),
			LAYERS_PRO_VER
		);
	}

	/**
	*  Enqueue Front End Scripts & Styles
	*/

	public function enqueue_scripts(){

		// Enqueue CSS
		wp_enqueue_style(
			LAYERS_PRO_SLUG . '-pro',
			LAYERS_PRO_URI . '/assets/css/layers-pro.css',
			array(),
			LAYERS_PRO_VER
		);

		// Enqueue Script
		wp_enqueue_script(
			LAYERS_PRO_SLUG . '-frontend',
			LAYERS_PRO_URI . 'assets/js/layers-pro.js',
			array( 'jquery' ),
			LAYERS_PRO_VER
		);

		// Enqueue Font-Awesome.
		wp_enqueue_style( 'layers-font-awesome' );
	}

	/**
	* Slider Background Elements
	*/

	public function slider_elements( $elements, $widget ){

		$new_elements = array();

		foreach( $elements as $key => $element ){

			$new_elements[ $key ] = $element;

			if( 'background-image' == $key ){
				$new_elements[ 'background-video-type' ] = array(
					'type' => 'select',
					'label' => __( 'Background Video', 'layers-pro' ),
					'name' => $widget->get_layers_field_name( 'background', 'video-type' ),
					'id' => $widget->get_layers_field_id( 'background', 'video-type' ),
					'value' => ( isset( $widget->values['background']['video-type'] ) ) ? $widget->values['background']['video-type'] : NULL,
					'options' => array(
						'' => __( 'None', 'layers-pro' ),
						'self-hosted' => __( 'Self Hosted', 'layers-pro' ),
						'youtube' => __( 'YouTube', 'layers-pro' ),
						'vimeo' => __( 'Vimeo', 'layers-pro' ),
					)
				);
				$new_elements[ 'background-video-mp4' ] = array(
					'type' => 'upload',
					'label' => __( 'Background MP4', 'layers-pro' ),
					'button_label' => __( 'Choose MP4 File', 'layers-pro' ),
					'name' => $widget->get_layers_field_name( 'background', 'video-mp4' ),
					'id' => $widget->get_layers_field_id( 'background', 'video-mp4' ),
					'value' => ( isset( $widget->values['background']['video-mp4'] ) ) ? $widget->values['background']['video-mp4'] : NULL,
					'data' => array(
						'show-if-selector' => '#' . $widget->get_layers_field_id( 'background', 'video-type' ),
						'show-if-value' => 'self-hosted'
					)
				);
				$new_elements[ 'background-video-youtube' ] = array(
					'type' => 'text',
					'label' => __( 'YouTube URL', 'layers-pro' ),
					'name' => $widget->get_layers_field_name( 'background', 'video-youtube' ),
					'id' => $widget->get_layers_field_id( 'background', 'video-youtube' ),
					'value' => ( isset( $widget->values['background']['video-youtube'] ) ) ? $widget->values['background']['video-youtube'] : NULL,
					'data' => array(
						'show-if-selector' => '#' . $widget->get_layers_field_id( 'background', 'video-type' ),
						'show-if-value' => 'youtube'
					)
				);
				$new_elements[ 'background-video-vimeo' ] = array(
					'type' => 'text',
					'label' => __( 'Vimeo URL', 'layers-pro' ),
					'name' => $widget->get_layers_field_name( 'background', 'video-vimeo' ),
					'id' => $widget->get_layers_field_id( 'background', 'video-vimeo' ),
					'value' => ( isset( $widget->values['background']['video-vimeo'] ) ) ? $widget->values['background']['video-vimeo'] : NULL,
					'data' => array(
						'show-if-selector' => '#' . $widget->get_layers_field_id( 'background', 'video-type' ),
						'show-if-value' => 'vimeo'
					)
				);
			}
		}

		return $new_elements;
	}

	/**
	* Slider Background Elements
	*/

	public function slider_video_bg( $widget_this, $item_instance, $widget_instance ){
		global $wp_customize;

		// Get the mobile fallback
		$bg_img = $widget_this->check_and_return( $item_instance, 'design', 'background', 'image');

		// Get the video type
		$bg_video_type = $widget_this->check_and_return( $item_instance, 'design', 'background', 'video-type');

		// Spool up the video URLs
		$mp4 = $widget_this->check_and_return( $item_instance, 'design', 'background', 'video-mp4');
		$youtube = $widget_this->check_and_return( $item_instance, 'design', 'background', 'video-youtube');
		$vimeo = $widget_this->check_and_return( $item_instance, 'design', 'background', 'video-vimeo');

		if( 'self-hosted' == $bg_video_type ) {
			if( !$wp_customize && 1 == count( $widget_this->check_and_return( $widget_instance, 'slides' ) ) ) {
				$autoplay = 'autoplay';
			} else {
				$autoplay = '';
			}
			?>
			<video <?php if( $wp_customize ) echo 'customizer'; ?> <?php echo $autoplay; ?> loop <?php if( $bg_img ) { ?>poster="<?php echo wp_get_attachment_url( $bg_img ); ?>"<?php } ?>>
				<?php if( $mp4 ) { ?>
					<source src="<?php echo wp_get_attachment_url( $mp4 ); ?>" type="video/mp4" />
  				<?php } ?>
			</video>
		<?php }
		elseif( function_exists( 'layers_get_youtube_id' ) && ( $youtube && 'youtube' == $bg_video_type ) || ( $vimeo && 'vimeo' == $bg_video_type ) ) {
			if( !$wp_customize && 1 == count( $widget_this->check_and_return( $widget_instance, 'slides' ) ) ) {
				$autoplay = '&autoplay=1';
			} else {
				$autoplay = '';
			} ?>
			<div class="layerspro-slider-video">
				<?php if( $youtube && 'youtube' == $bg_video_type ) {
					$id = layers_get_youtube_id( $youtube ) ?>
					<iframe frameborder="0" height="100%" width="100%" src="//www.youtube.com/embed/<?php echo $id; ?>?enablejsapi=1&controls=0&loop=1&rel=0&showinfo=0&autohide=1&wmode=transparent&hd=1<?php echo $autoplay; ?>"></iframe>
				<?php } elseif( $vimeo && 'vimeo' == $bg_video_type ) {
					$id = layers_get_vimeo_id( $vimeo ); ?>
					<iframe frameborder="0" height="100%" width="100%" src="//player.vimeo.com/video/<?php echo $id; ?>?title=0&controls=0&byline=0&portrait=0&loop=1<?php echo $autoplay; ?>" webkitallowfullscreen mozallowfullscreen allowfullscreen></iframe>
				<?php }  ?>
			</div>
		<?php }
	}

}
